@@ -5,14 +5,17 @@
 // http://opensource.org/licenses/MIT>, at your option. This file may not be
 // copied, modified, or distributed except according to those terms.
 
-//! Implementation of different persistent storages. Currently on disk (`FileStorage`) and in Redis
-//! (`RedisStorage`).
+//! Implementation of different persistent storages. Currently on disk (`FileStorage` and
+//! `FileMemoryStorage`) and in Redis (`RedisStorage`).
 
-/// `RedisStorage`
-pub mod redis;
 /// `FileStorage`
 pub mod file;
+/// `FileMemoryStorage`
+pub mod file_memory;
+/// `RedisStorage`
+pub mod redis;
 
+pub use storage::file::FileStorage;
+pub use storage::file_memory::FileMemoryStorage;
 /// Bring them into scope
 pub use storage::redis::RedisStorage;
-pub use storage::file::FileStorage;
